@@ -1,4 +1,5 @@
 # pylint: disable=too-many-lines,redefined-outer-name
+import base64
 import enum
 import os
 import sys
@@ -6,7 +7,7 @@ import traceback
 from collections import defaultdict
 from enum import Enum
 from queue import Queue
-from typing import Any, Callable, Dict, Generator, Iterable, List, NoReturn, Optional, Tuple, Union
+from typing import Any, Callable, Dict, Generator, Iterable, List, NoReturn, Optional, Tuple, Union, cast
 from urllib.parse import urlparse
 
 import attr
@@ -20,10 +21,13 @@ from .. import fixups as _fixups
 from .. import runner, service
 from .. import targets as targets_module
 from ..constants import (
+    API_NAME_ENV_VAR,
+    BASE_URL_ENV_VAR,
     DEFAULT_DATA_GENERATION_METHODS,
     DEFAULT_RESPONSE_TIMEOUT,
     DEFAULT_STATEFUL_RECURSION_LIMIT,
     HYPOTHESIS_IN_MEMORY_DATABASE_IDENTIFIER,
+    WAIT_FOR_SCHEMA_ENV_VAR,
     CodeSampleStyle,
     DataGenerationMethod,
 )
@@ -39,10 +43,10 @@ from ..specs.openapi import loaders as oas_loaders
 from ..stateful import Stateful
 from ..targets import Target
 from ..types import Filter, PathLike, RequestCert
-from ..utils import GenericResponse, file_exists, get_requests_auth, import_app
+from ..utils import current_datetime, file_exists, get_requests_auth, import_app
 from . import callbacks, cassettes, output
 from .constants import DEFAULT_WORKERS, MAX_WORKERS, MIN_WORKERS
-from .context import ExecutionContext, ServiceContext
+from .context import ExecutionContext, FileReportContext, ServiceReportContext
 from .debug import DebugOutputHandler
 from .handlers import EventHandler
 from .junitxml import JunitXMLHandler
@@ -69,7 +73,7 @@ DEFAULT_TARGETS_NAMES = _get_callable_names(targets_module.DEFAULT_TARGETS)
 ALL_TARGETS_NAMES = _get_callable_names(targets_module.ALL_TARGETS)
 TARGETS_TYPE = click.Choice((*ALL_TARGETS_NAMES, "all"))
 
-DATA_GENERATION_METHOD_TYPE = click.Choice([item.name for item in DataGenerationMethod])
+DATA_GENERATION_METHOD_TYPE = click.Choice([item.name for item in DataGenerationMethod] + ["all"])
 
 DEPRECATED_CASSETTE_PATH_OPTION_WARNING = (
     "Warning: Option `--store-network-log` is deprecated and will be removed in Schemathesis 4.0. "
@@ -188,7 +192,7 @@ with_request_tls_verify = click.option(
     type=str,
     default="true",
     show_default=True,
-    callback=callbacks.convert_request_tls_verify,
+    callback=callbacks.convert_boolean_string,
 )
 with_request_cert = click.option(
     "--request-cert",
@@ -213,12 +217,14 @@ with_hosts_file = click.option(
     type=click.Path(dir_okay=False, writable=True),
     default=service.DEFAULT_HOSTS_PATH,
     envvar=service.HOSTS_PATH_ENV_VAR,
+    callback=callbacks.convert_hosts_file,
 )
+REPORT_TO_SERVICE = object()
 
 
 @schemathesis.command(short_help="Perform schemathesis test.", cls=CommandWithCustomHelp)
 @click.argument("schema", type=str)
-@click.argument("api_slug", type=str, required=False)
+@click.argument("api_name", type=str, required=False, envvar=API_NAME_ENV_VAR)
 @click.option(
     "--checks",
     "-c",
@@ -267,6 +273,13 @@ with_hosts_file = click.option(
     help="Exit instantly on first error or failed test.",
     show_default=True,
 )
+@click.option(
+    "--max-failures",
+    "max_failures",
+    type=click.IntRange(min=1),
+    help="Exit after N failures or errors.",
+    show_default=True,
+)
 @click.option(
     "--dry-run",
     "dry_run",
@@ -358,8 +371,16 @@ with_hosts_file = click.option(
     help="Base URL address of the API, required for SCHEMA if specified by file.",
     type=str,
     callback=callbacks.validate_base_url,
+    envvar=BASE_URL_ENV_VAR,
 )
 @click.option("--app", help="WSGI/ASGI application to test.", type=str, callback=callbacks.validate_app)
+@click.option(
+    "--wait-for-schema",
+    help="Maximum time in seconds to wait on the API schema availability.",
+    type=click.FloatRange(1.0),
+    default=None,
+    envvar=WAIT_FOR_SCHEMA_ENV_VAR,
+)
 @click.option(
     "--request-timeout",
     help="Timeout in milliseconds for network requests during the test run.",
@@ -391,6 +412,15 @@ with_hosts_file = click.option(
 @click.option(
     "--junit-xml", help="Create junit-xml style report file at given path.", type=click.File("w", encoding="utf-8")
 )
+@click.option(
+    "--report",
+    help="Upload test report to Schemathesis.io, or store in a file.",
+    is_flag=False,
+    flag_value=REPORT_TO_SERVICE,
+    type=click.File("wb"),
+    envvar=service.REPORT_ENV_VAR,
+    callback=callbacks.convert_report,  # type: ignore
+)
 @click.option(
     "--debug-output-file",
     help="Save debug output as JSON lines in the given file.",
@@ -539,13 +569,22 @@ with_hosts_file = click.option(
     type=str,
     envvar=service.URL_ENV_VAR,
 )
+@click.option(
+    "--schemathesis-io-telemetry",
+    help="Controls whether you send anonymized CLI usage data to Schemathesis.io along with your report.",
+    type=str,
+    default="true",
+    show_default=True,
+    callback=callbacks.convert_boolean_string,
+    envvar=service.TELEMETRY_ENV_VAR,
+)
 @with_hosts_file
-@click.option("--verbosity", "-v", help="Reduce verbosity of error output.", count=True)
+@click.option("--verbosity", "-v", help="Increase verbosity of the output.", count=True)
 @click.pass_context
 def run(
     ctx: click.Context,
     schema: str,
-    api_slug: Optional[str],
+    api_name: Optional[str],
     auth: Optional[Tuple[str, str]],
     auth_type: str,
     headers: Dict[str, str],
@@ -554,6 +593,7 @@ def run(
     max_response_time: Optional[int] = None,
     targets: Iterable[str] = DEFAULT_TARGETS_NAMES,
     exit_first: bool = False,
+    max_failures: Optional[int] = None,
     dry_run: bool = False,
     endpoints: Optional[Filter] = None,
     methods: Optional[Filter] = None,
@@ -575,6 +615,7 @@ def run(
     cassette_path: Optional[click.utils.LazyFile] = None,
     cassette_preserve_exact_body_bytes: bool = False,
     store_network_log: Optional[click.utils.LazyFile] = None,
+    wait_for_schema: Optional[float] = None,
     fixups: Tuple[str] = (),  # type: ignore
     stateful: Optional[Stateful] = None,
     stateful_recursion_limit: int = DEFAULT_STATEFUL_RECURSION_LIMIT,
@@ -590,17 +631,20 @@ def run(
     hypothesis_verbosity: Optional[hypothesis.Verbosity] = None,
     verbosity: int = 0,
     no_color: bool = False,
+    report: Optional[click.utils.LazyFile] = None,
     schemathesis_io_token: Optional[str] = None,
     schemathesis_io_url: str = service.DEFAULT_URL,
+    schemathesis_io_telemetry: bool = True,
     hosts_file: PathLike = service.DEFAULT_HOSTS_PATH,
 ) -> None:
     """Perform schemathesis test against an API specified by SCHEMA.
 
     SCHEMA must be a valid URL or file path pointing to an Open API / GraphQL specification.
 
-    API_SLUG is an API identifier to upload data to Schemathesis.io.
+    API_NAME is an API identifier to upload data to Schemathesis.io.
     """
-    # pylint: disable=too-many-locals
+    # pylint: disable=too-many-locals,too-many-branches
+    started_at = current_datetime()
     maybe_disable_color(ctx, no_color)
     check_auth(auth, headers)
     selected_targets = tuple(target for target in targets_module.ALL_TARGETS if target.__name__ in targets)
@@ -615,32 +659,44 @@ def run(
     schemathesis_io_hostname = urlparse(schemathesis_io_url).netloc
     token = schemathesis_io_token or service.hosts.get_token(hostname=schemathesis_io_hostname, hosts_file=hosts_file)
     schema_kind = callbacks.parse_schema_kind(schema, app)
-    callbacks.validate_schema(schema, schema_kind, base_url=base_url, dry_run=dry_run, app=app, api_slug=api_slug)
+    callbacks.validate_schema(schema, schema_kind, base_url=base_url, dry_run=dry_run, app=app, api_name=api_name)
     client = None
-    test_run = None
-    if api_slug is not None or schema_kind == callbacks.SchemaInputKind.SLUG:
-        if token is None:
-            hostname = (
-                "Schemathesis.io" if schemathesis_io_hostname == service.DEFAULT_HOSTNAME else schemathesis_io_hostname
-            )
-            raise click.UsageError(
-                "\n\n"
-                f"You are trying to upload data to {hostname}, but your CLI appears to be not authenticated.\n\n"
-                "To authenticate, grab your token from `app.schemathesis.io` and run `st auth login <TOKEN>`\n"
-                "Alternatively, you can pass the token explicitly via the `--schemathesis-io-token` option / "
-                f"`{service.TOKEN_ENV_VAR}` environment variable\n\n"
-                "See https://schemathesis.readthedocs.io/en/stable/service.html for more details"
-            )
+    if schema_kind == callbacks.SchemaInputKind.NAME:
+        api_name = schema
+    if (
+        not (report and report.name is not REPORT_TO_SERVICE)
+        and api_name is not None
+        and schema_kind == callbacks.SchemaInputKind.NAME
+    ):
         client = service.ServiceClient(base_url=schemathesis_io_url, token=token)
-        try:
-            test_run = client.create_test_run(schema)
-            if schema_kind == callbacks.SchemaInputKind.SLUG:
+        # It is assigned above
+        if token is not None or schema_kind == callbacks.SchemaInputKind.NAME:
+            if token is None:
+                hostname = (
+                    "Schemathesis.io"
+                    if schemathesis_io_hostname == service.DEFAULT_HOSTNAME
+                    else schemathesis_io_hostname
+                )
+                raise click.UsageError(
+                    "\n\n"
+                    f"You are trying to upload data to {hostname}, but your CLI appears to be not authenticated.\n\n"
+                    "To authenticate, grab your token from `app.schemathesis.io` and run `st auth login <TOKEN>`\n"
+                    "Alternatively, you can pass the token explicitly via the `--schemathesis-io-token` option / "
+                    f"`{service.TOKEN_ENV_VAR}` environment variable\n\n"
+                    "See https://schemathesis.readthedocs.io/en/stable/service.html for more details"
+                )
+            name: str = cast(str, api_name)
+            try:
+                details = client.get_api_details(name)
                 # Replace config values with ones loaded from the service
-                schema = test_run.config.location
-                base_url = base_url or test_run.config.base_url
-        except requests.HTTPError as exc:
-            handle_service_error(exc)
-
+                schema = details.location
+                base_url = base_url or details.base_url
+            except requests.HTTPError as exc:
+                handle_service_error(exc, name)
+    if report and report.name is REPORT_TO_SERVICE and not client:
+        # Upload without connecting data to a certain API
+        client = service.ServiceClient(base_url=schemathesis_io_url, token=token)
+    host_data = service.hosts.HostData(schemathesis_io_hostname, hosts_file)
     if "all" in checks:
         selected_checks = checks_module.ALL_CHECKS
     else:
@@ -665,12 +721,14 @@ def run(
         schema,
         app=app,
         base_url=base_url,
+        started_at=started_at,
         validate_schema=validate_schema,
         skip_deprecated_operations=skip_deprecated_operations,
         data_generation_methods=data_generation_methods,
         force_schema_version=force_schema_version,
         request_tls_verify=request_tls_verify,
         request_cert=prepare_request_cert(request_cert, request_cert_key),
+        wait_for_schema=wait_for_schema,
         auth=auth,
         auth_type=auth_type,
         headers=headers,
@@ -681,6 +739,7 @@ def run(
         request_timeout=request_timeout,
         seed=hypothesis_seed,
         exit_first=exit_first,
+        max_failures=max_failures,
         dry_run=dry_run,
         store_interactions=cassette_path is not None,
         checks=selected_checks,
@@ -693,19 +752,25 @@ def run(
     )
     execute(
         event_stream,
-        hypothesis_settings,
-        workers_num,
-        show_errors_tracebacks,
-        validate_schema,
-        cassette_path,
-        cassette_preserve_exact_body_bytes,
-        junit_xml,
-        verbosity,
-        code_sample_style,
-        debug_output_file,
-        schemathesis_io_url,
-        client,
-        test_run,
+        hypothesis_settings=hypothesis_settings,
+        workers_num=workers_num,
+        show_errors_tracebacks=show_errors_tracebacks,
+        validate_schema=validate_schema,
+        cassette_path=cassette_path,
+        cassette_preserve_exact_body_bytes=cassette_preserve_exact_body_bytes,
+        junit_xml=junit_xml,
+        verbosity=verbosity,
+        code_sample_style=code_sample_style,
+        data_generation_methods=data_generation_methods,
+        debug_output_file=debug_output_file,
+        host_data=host_data,
+        client=client,
+        report=report,
+        telemetry=schemathesis_io_telemetry,
+        api_name=api_name,
+        location=schema,
+        base_url=base_url,
+        started_at=started_at,
     )
 
 
@@ -731,6 +796,7 @@ class LoaderConfig:
     force_schema_version: Optional[str] = attr.ib()  # pragma: no mutate
     request_tls_verify: Union[bool, str] = attr.ib()  # pragma: no mutate
     request_cert: Optional[RequestCert] = attr.ib()  # pragma: no mutate
+    wait_for_schema: Optional[float] = attr.ib()  # pragma: no mutate
     # Network request parameters
     auth: Optional[Tuple[str, str]] = attr.ib()  # pragma: no mutate
     auth_type: Optional[str] = attr.ib()  # pragma: no mutate
@@ -747,6 +813,7 @@ def into_event_stream(
     *,
     app: Any,
     base_url: Optional[str],
+    started_at: str,
     validate_schema: bool,
     skip_deprecated_operations: bool,
     data_generation_methods: Tuple[DataGenerationMethod, ...],
@@ -758,6 +825,7 @@ def into_event_stream(
     auth_type: Optional[str],
     headers: Optional[Dict[str, str]],
     request_timeout: Optional[int],
+    wait_for_schema: Optional[float],
     # Schema filters
     endpoint: Optional[Filter],
     method: Optional[Filter],
@@ -771,6 +839,7 @@ def into_event_stream(
     hypothesis_settings: Optional[hypothesis.settings],
     seed: Optional[int],
     exit_first: bool,
+    max_failures: Optional[int],
     dry_run: bool,
     store_interactions: bool,
     stateful: Optional[Stateful],
@@ -789,6 +858,7 @@ def into_event_stream(
             force_schema_version=force_schema_version,
             request_tls_verify=request_tls_verify,
             request_cert=request_cert,
+            wait_for_schema=wait_for_schema,
             auth=auth,
             auth_type=auth_type,
             headers=headers,
@@ -808,6 +878,8 @@ def into_event_stream(
             request_cert=request_cert,
             seed=seed,
             exit_first=exit_first,
+            max_failures=max_failures,
+            started_at=started_at,
             dry_run=dry_run,
             store_interactions=store_interactions,
             checks=checks,
@@ -819,7 +891,7 @@ def into_event_stream(
             hypothesis_settings=hypothesis_settings,
         ).execute()
     except Exception as exc:
-        yield events.InternalError.from_exc(exc)
+        yield events.InternalError.from_exc(exc, wait_for_schema)
 
 
 def load_schema(config: LoaderConfig) -> BaseSchema:
@@ -919,6 +991,8 @@ def _add_requests_kwargs(kwargs: Dict[str, Any], config: LoaderConfig) -> None:
         kwargs["cert"] = config.request_cert
     if config.auth is not None:
         kwargs["auth"] = get_requests_auth(config.auth, config.auth_type)
+    if config.wait_for_schema is not None:
+        kwargs["wait_for_schema"] = config.wait_for_schema
 
 
 def is_probably_graphql(location: str) -> bool:
@@ -960,6 +1034,7 @@ class OutputStyle(Enum):
 
 def execute(
     event_stream: Generator[events.ExecutionEvent, None, None],
+    *,
     hypothesis_settings: hypothesis.settings,
     workers_num: int,
     show_errors_tracebacks: bool,
@@ -969,19 +1044,52 @@ def execute(
     junit_xml: Optional[click.utils.LazyFile],
     verbosity: int,
     code_sample_style: CodeSampleStyle,
+    data_generation_methods: Tuple[DataGenerationMethod, ...],
     debug_output_file: Optional[click.utils.LazyFile],
-    schemathesis_io_url: str,
+    host_data: service.hosts.HostData,
     client: Optional[service.ServiceClient],
-    test_run: Optional[service.TestRun],
+    report: Optional[click.utils.LazyFile],
+    telemetry: bool,
+    api_name: Optional[str],
+    location: str,
+    base_url: Optional[str],
+    started_at: str,
 ) -> None:
     """Execute a prepared runner by drawing events from it and passing to a proper handler."""
+    # pylint: disable=too-many-branches
     handlers: List[EventHandler] = []
-    service_context = None
-    if client is not None and test_run is not None:
-        service_queue: Queue = Queue()
-        service_context = ServiceContext(url=schemathesis_io_url, queue=service_queue)
-        reporter = service.ServiceReporter(client=client, test_run=test_run, out_queue=service_queue)
-        handlers.append(reporter)
+    report_context: Optional[Union[ServiceReportContext, FileReportContext]] = None
+    report_queue: Queue
+    if client:
+        # If API name is specified, validate it
+        report_queue = Queue()
+        report_context = ServiceReportContext(queue=report_queue, service_base_url=client.base_url)
+        handlers.append(
+            service.ServiceReportHandler(
+                client=client,
+                host_data=host_data,
+                api_name=api_name,
+                location=location,
+                base_url=base_url,
+                started_at=started_at,
+                out_queue=report_queue,
+                telemetry=telemetry,
+            )
+        )
+    elif report and report.name is not REPORT_TO_SERVICE:
+        report_queue = Queue()
+        report_context = FileReportContext(queue=report_queue, filename=report.name)
+        handlers.append(
+            service.FileReportHandler(
+                file_handle=report,
+                api_name=api_name,
+                location=location,
+                base_url=base_url,
+                started_at=started_at,
+                out_queue=report_queue,
+                telemetry=telemetry,
+            )
+        )
     if junit_xml is not None:
         handlers.append(JunitXMLHandler(junit_xml))
     if debug_output_file is not None:
@@ -1001,7 +1109,7 @@ def execute(
         junit_xml_file=junit_xml.name if junit_xml is not None else None,
         verbosity=verbosity,
         code_sample_style=code_sample_style,
-        service=service_context,
+        report=report_context,
     )
 
     def shutdown() -> None:
@@ -1029,9 +1137,11 @@ def execute(
     sys.exit(1)
 
 
-def handle_service_error(exc: requests.HTTPError) -> NoReturn:
-    if exc.response.status_code == 404:
-        error_message("API_SLUG not found!")
+def handle_service_error(exc: requests.HTTPError, api_name: str) -> NoReturn:
+    if exc.response.status_code == 403:
+        error_message(exc.response.json()["detail"])
+    elif exc.response.status_code == 404:
+        error_message(f"API with name `{api_name}` not found!")
     else:
         output.default.display_service_error(service.Error(exc))
     sys.exit(1)
@@ -1054,6 +1164,7 @@ def get_exit_code(event: events.ExecutionEvent) -> int:
 @click.option("--uri", help="A regexp that filters interactions by their request URI.", type=str)
 @click.option("--method", help="A regexp that filters interactions by their request method.", type=str)
 @click.option("--no-color", help="Disable ANSI color escape codes.", type=bool, is_flag=True)
+@click.option("--verbosity", "-v", help="Increase verbosity of the output.", count=True)
 @with_request_tls_verify
 @with_request_cert
 @with_request_cert_key
@@ -1066,6 +1177,7 @@ def replay(
     uri: Optional[str] = None,
     method: Optional[str] = None,
     no_color: bool = False,
+    verbosity: int = 0,
     request_tls_verify: bool = True,
     request_cert: Optional[str] = None,
     request_cert_key: Optional[str] = None,
@@ -1092,7 +1204,21 @@ def replay(
         click.secho(f"  {bold('ID')}              : {replayed.interaction['id']}")
         click.secho(f"  {bold('URI')}             : {replayed.interaction['request']['uri']}")
         click.secho(f"  {bold('Old status code')} : {replayed.interaction['response']['status']['code']}")
-        click.secho(f"  {bold('New status code')} : {replayed.response.status_code}\n")
+        click.secho(f"  {bold('New status code')} : {replayed.response.status_code}")
+        if verbosity > 0:
+            data = replayed.interaction["response"]
+            old_body = ""
+            # Body may be missing for 204 responses
+            if "body" in data:
+                if "base64_string" in data["body"]:
+                    content = data["body"]["base64_string"]
+                    if content:
+                        old_body = base64.b64decode(content).decode(errors="replace")
+                else:
+                    old_body = data["body"]["string"]
+            click.secho(f"  {bold('Old payload')} : {old_body}")
+            click.secho(f"  {bold('New payload')} : {replayed.response.text}")
+        click.echo()
 
 
 @schemathesis.group(short_help="Authenticate Schemathesis.io.")
@@ -1189,28 +1315,6 @@ def after_init_cli_run_handlers(
     """
 
 
-@HookDispatcher.register_spec([HookScope.GLOBAL])
-def before_call(context: HookContext, case: Case) -> None:
-    """Called before every network call in CLI tests.
-
-    Use cases:
-     - Modification of `case`. For example, adding some pre-determined value to its query string.
-     - Logging
-    """
-
-
-@HookDispatcher.register_spec([HookScope.GLOBAL])
-def after_call(context: HookContext, case: Case, response: GenericResponse) -> None:
-    """Called after every network call in CLI tests.
-
-    Note that you need to modify the response in-place.
-
-    Use cases:
-     - Response post-processing, like modifying its payload.
-     - Logging
-    """
-
-
 @HookDispatcher.register_spec([HookScope.GLOBAL])
 def process_call_kwargs(context: HookContext, case: Case, kwargs: Dict[str, Any]) -> None:
     """Called before every network call in CLI tests.
