@@ -7,17 +7,20 @@ from urllib.parse import urlparse
 
 import click
 import hypothesis
+from click.types import LazyFile  # type: ignore
 from requests import PreparedRequest, RequestException
 
 from .. import utils
 from ..constants import CodeSampleStyle, DataGenerationMethod
+from ..service.hosts import get_temporary_hosts_file
 from ..stateful import Stateful
+from ..types import PathLike
 from .constants import DEFAULT_WORKERS
 
 MISSING_CASSETTE_PATH_ARGUMENT_MESSAGE = (
     'Missing argument, "--cassette-path" should be specified as well if you use "--cassette-preserve-exact-body-bytes".'
 )
-INVALID_SCHEMA_MESSAGE = "Invalid SCHEMA, must be a valid URL, file path or an API slug from Schemathesis.io."
+INVALID_SCHEMA_MESSAGE = "Invalid SCHEMA, must be a valid URL, file path or an API name from Schemathesis.io."
 
 
 @enum.unique
@@ -30,8 +33,8 @@ class SchemaInputKind(enum.Enum):
     PATH = 2
     # Relative path within a Python app
     APP_PATH = 3
-    # A short name for API created in Schemathesis.io
-    SLUG = 4
+    # A name for API created in Schemathesis.io
+    NAME = 4
 
 
 def parse_schema_kind(schema: str, app: Optional[str]) -> SchemaInputKind:
@@ -48,8 +51,8 @@ def parse_schema_kind(schema: str, app: Optional[str]) -> SchemaInputKind:
         return SchemaInputKind.PATH
     if app is not None:
         return SchemaInputKind.APP_PATH
-    # Assume SLUG if it is not a URL or PATH or APP_PATH
-    return SchemaInputKind.SLUG
+    # Assume NAME if it is not a URL or PATH or APP_PATH
+    return SchemaInputKind.NAME
 
 
 def validate_schema(
@@ -59,7 +62,7 @@ def validate_schema(
     base_url: Optional[str],
     dry_run: bool,
     app: Optional[str],
-    api_slug: Optional[str],
+    api_name: Optional[str],
 ) -> None:
     if kind == SchemaInputKind.URL:
         validate_url(schema)
@@ -67,9 +70,9 @@ def validate_schema(
         # Base URL is required if it is not a dry run
         if app is None and base_url is None and not dry_run:
             raise click.UsageError('Missing argument, "--base-url" is required for SCHEMA specified by file.')
-    if kind == SchemaInputKind.SLUG:
-        if api_slug is not None:
-            raise click.UsageError(f"Got unexpected extra argument ({api_slug})")
+    if kind == SchemaInputKind.NAME:
+        if api_name is not None:
+            raise click.UsageError(f"Got unexpected extra argument ({api_name})")
 
 
 def validate_url(value: str) -> None:
@@ -203,17 +206,49 @@ def convert_code_sample_style(ctx: click.core.Context, param: click.core.Paramet
 def convert_data_generation_method(
     ctx: click.core.Context, param: click.core.Parameter, value: str
 ) -> List[DataGenerationMethod]:
+    if value == "all":
+        return DataGenerationMethod.all()
     return [DataGenerationMethod[value]]
 
 
-def convert_request_tls_verify(ctx: click.core.Context, param: click.core.Parameter, value: str) -> Union[str, bool]:
-    if value.lower() in ("y", "yes", "t", "true", "on", "1"):
+TRUE_VALUES = ("y", "yes", "t", "true", "on", "1")
+FALSE_VALUES = ("n", "no", "f", "false", "off", "0")
+
+
+def _is_usable_dir(path: PathLike) -> bool:
+    if os.path.isfile(path):
+        path = os.path.dirname(path)
+    while not os.path.exists(path):
+        path = os.path.dirname(path)
+    return os.path.isdir(path) and os.access(path, os.R_OK | os.W_OK | os.X_OK)
+
+
+def convert_hosts_file(ctx: click.core.Context, param: click.core.Parameter, value: PathLike) -> PathLike:
+    if not _is_usable_dir(value):
+        path = get_temporary_hosts_file()
+        click.secho(
+            "WARNING: The provided hosts.toml file location is unusable - using a temporary file for this session. "
+            f"path={str(value)!r}",
+            fg="yellow",
+        )
+        return path
+    return value
+
+
+def convert_boolean_string(ctx: click.core.Context, param: click.core.Parameter, value: str) -> Union[str, bool]:
+    if value.lower() in TRUE_VALUES:
         return True
-    if value.lower() in ("n", "no", "f", "false", "off", "0"):
+    if value.lower() in FALSE_VALUES:
         return False
     return value
 
 
+def convert_report(ctx: click.core.Context, param: click.core.Option, value: LazyFile) -> LazyFile:
+    if param.resolve_envvar_value(ctx) is not None and value.name.lower() in TRUE_VALUES:
+        value.name = param.flag_value
+    return value
+
+
 @contextmanager
 def reraise_format_error(raw_value: str) -> Generator[None, None, None]:
     try:
