@@ -1,7 +1,7 @@
 import os
 import shutil
 from queue import Queue
-from typing import List, Optional
+from typing import List, Optional, Union
 
 import attr
 import hypothesis
@@ -11,9 +11,15 @@ from ..runner.serialization import SerializedTestResult
 
 
 @attr.s(slots=True)  # pragma: no mutate
-class ServiceContext:
-    url: str = attr.ib()  # pragma: no mutate
+class ServiceReportContext:
     queue: Queue = attr.ib()  # pragma: no mutate
+    service_base_url: str = attr.ib()  # pragma: no mutate
+
+
+@attr.s(slots=True)  # pragma: no mutate
+class FileReportContext:
+    queue: Queue = attr.ib()  # pragma: no mutate
+    filename: str = attr.ib(default=None)  # pragma: no mutate
 
 
 @attr.s(slots=True)  # pragma: no mutate
@@ -33,6 +39,7 @@ class ExecutionContext:
     results: List[SerializedTestResult] = attr.ib(factory=list)  # pragma: no mutate
     cassette_path: Optional[str] = attr.ib(default=None)  # pragma: no mutate
     junit_xml_file: Optional[str] = attr.ib(default=None)  # pragma: no mutate
+    is_interrupted: bool = attr.ib(default=False)  # pragma: no mutate
     verbosity: int = attr.ib(default=0)  # pragma: no mutate
     code_sample_style: CodeSampleStyle = attr.ib(default=CodeSampleStyle.default())  # pragma: no mutate
-    service: Optional[ServiceContext] = attr.ib(default=None)  # pragma: no mutate
+    report: Optional[Union[ServiceReportContext, FileReportContext]] = attr.ib(default=None)  # pragma: no mutate
