@@ -15,7 +15,7 @@ from ...constants import DISCORD_LINK, FLAKY_FAILURE_MESSAGE, CodeSampleStyle, _
 from ...models import Response, Status
 from ...runner import events
 from ...runner.serialization import SerializedCase, SerializedError, SerializedTestResult, deduplicate_failures
-from ..context import ExecutionContext
+from ..context import ExecutionContext, FileReportContext, ServiceReportContext
 from ..handlers import EventHandler
 
 DISABLE_SCHEMA_VALIDATION_MESSAGE = (
@@ -35,16 +35,15 @@ def get_terminal_width() -> int:
     return shutil.get_terminal_size((80, 24)).columns
 
 
-def display_section_name(title: str, separator: str = "=", extra: str = "", **kwargs: Any) -> None:
+def display_section_name(title: str, separator: str = "=", **kwargs: Any) -> None:
     """Print section name with separators in terminal with the given title nicely centered."""
-    extra = extra if not extra else f" [{extra}]"
-    message = f" {title}{extra} ".center(get_terminal_width(), separator)
+    message = f" {title} ".center(get_terminal_width(), separator)
     kwargs.setdefault("bold", True)
     click.secho(message, **kwargs)
 
 
 def display_subsection(result: SerializedTestResult, color: Optional[str] = "red") -> None:
-    display_section_name(result.verbose_name, "_", result.data_generation_method, fg=color)
+    display_section_name(result.verbose_name, "_", fg=color)
 
 
 def get_percentage(position: int, length: int) -> str:
@@ -237,12 +236,9 @@ def display_example(
             message = reduce_schema_error(message)
         click.secho(message, fg="red")
         click.echo()
-    for line in case.text_lines:
-        click.secho(line, fg="red")
-    click.echo()
     if response is not None and response.body is not None:
         payload = base64.b64decode(response.body).decode(response.encoding or "utf8", errors="replace")
-        click.secho(f"----------\n\nResponse status: {response.status_code}\nResponse payload: `{payload}`\n", fg="red")
+        click.secho(f"Response status: {response.status_code}\nResponse payload: `{payload}`\n", fg="red")
     if context.code_sample_style == CodeSampleStyle.python:
         click.secho(f"Run this Python code to reproduce this failure: \n\n    {case.requests_code}\n", fg="red")
     if context.code_sample_style == CodeSampleStyle.curl:
@@ -288,29 +284,71 @@ def display_statistic(context: ExecutionContext, event: events.Finished) -> None
         category = click.style("JUnit XML file", bold=True)
         click.secho(f"{category}: {context.junit_xml_file}")
 
-    handle_service_integration(context)
-
-
-def handle_service_integration(context: ExecutionContext) -> None:
-    """If Schemathesis.io integration is enabled, wait for the handler & print the resulting status."""
-    if context.service:
+    if event.warnings:
         click.echo()
-        title = click.style("Schemathesis.io", bold=True)
-        event = wait_for_service_handler(context.service.queue, title)
-        color = {
-            service.Completed: "green",
-            service.Error: "red",
-            service.Timeout: "red",
-        }[event.__class__]
-        status = click.style(event.name, fg=color, bold=True)
-        click.echo(f"{title}: {status}\r", nl=False)
-        click.echo()
-        if isinstance(event, service.Completed):
-            report_title = click.style("Report", bold=True)
-            click.echo(f"{report_title}: {event.short_url}")
-        if isinstance(event, service.Error):
+        if len(event.warnings) == 1:
+            title = click.style("WARNING:", bold=True, fg="yellow")
+            warning = click.style(event.warnings[0], fg="yellow")
+            click.secho(f"{title} {warning}")
+        else:
+            click.secho("WARNINGS:", bold=True, fg="yellow")
+            for warning in event.warnings:
+                click.secho(f"  - {warning}", fg="yellow")
+
+    if context.report is not None and not context.is_interrupted:
+        if isinstance(context.report, FileReportContext):
             click.echo()
-            display_service_error(event)
+            display_report_metadata(context.report.queue.get())
+            click.secho(f"Report is saved to {context.report.filename}", bold=True)
+        elif isinstance(context.report, ServiceReportContext):
+            click.echo()
+            handle_service_integration(context.report)
+    else:
+        click.echo()
+        category = click.style("Hint", bold=True)
+        click.echo(
+            f"{category}: You can visualize test results in Schemathesis.io by using `--report` in your CLI command."
+        )
+
+
+def handle_service_integration(context: ServiceReportContext) -> None:
+    """If Schemathesis.io integration is enabled, wait for the handler & print the resulting status."""
+    event = context.queue.get()
+    title = click.style("Upload", bold=True)
+    if isinstance(event, service.Metadata):
+        display_report_metadata(event)
+        click.secho(f"Uploading reports to {context.service_base_url} ...", bold=True)
+        event = wait_for_report_handler(context.queue, title)
+    color = {
+        service.Completed: "green",
+        service.Error: "red",
+        service.Failed: "red",
+        service.Timeout: "red",
+    }[event.__class__]
+    status = click.style(event.status, fg=color, bold=True)
+    click.echo(f"{title}: {status}\r", nl=False)
+    click.echo()
+    if isinstance(event, service.Error):
+        click.echo()
+        display_service_error(event)
+    if isinstance(event, service.Failed):
+        click.echo()
+        click.echo(event.detail)
+    if isinstance(event, service.Completed):
+        click.echo()
+        click.echo(event.message)
+        click.echo()
+        click.echo(event.next_url)
+
+
+def display_report_metadata(meta: service.Metadata) -> None:
+    if meta.ci_environment is not None:
+        click.secho(f"{meta.ci_environment.verbose_name} detected:", bold=True)
+        for key, value in meta.ci_environment.as_env().items():
+            if value is not None:
+                click.secho(f"  -> {key}: {value}")
+        click.echo()
+    click.secho(f"Compressed report size: {meta.size / 1024.:,.0f} KB", bold=True)
 
 
 def display_service_error(event: service.Error) -> None:
@@ -341,11 +379,14 @@ def display_service_error(event: service.Error) -> None:
         ask_to_report(event)
 
 
+SERVICE_ERROR_MESSAGE = "An error happened during uploading reports to Schemathesis.io"
+
+
 def ask_to_report(event: service.Error, report_to_issues: bool = True, extra: str = "") -> None:
     # Likely an internal Schemathesis error
-    message = event.get_message(True)
+    traceback = event.get_message(True)
     if isinstance(event.exception, requests.RequestException) and event.exception.response is not None:
-        response = f"Response: {event.exception.response.text}"
+        response = f"Response: {event.exception.response.text}\n"
     else:
         response = ""
     if report_to_issues:
@@ -353,20 +394,16 @@ def ask_to_report(event: service.Error, report_to_issues: bool = True, extra: st
     else:
         ask = ""
     click.secho(
-        f"An error happened during uploading reports to Schemathesis.io:\n"
-        f"{extra}"
-        f"{ask}"
-        f"{response}"
-        f"\n    {message.strip()}",
+        f"{SERVICE_ERROR_MESSAGE}:\n{extra}{ask}{response}\n{traceback.strip()}",
         fg="red",
     )
 
 
-def wait_for_service_handler(queue: Queue, title: str) -> service.Event:
+def wait_for_report_handler(queue: Queue, title: str) -> service.Event:
     """Wait for the Schemathesis.io handler to finish its job."""
     start = time.monotonic()
     spinner = create_spinner(SPINNER_REPETITION_NUMBER)
-    # The testing process it done and we need to wait for the Schemathesis.io handler to finish
+    # The testing process is done, and we need to wait for the Schemathesis.io handler to finish
     # It might still have some data to send
     while queue.empty():
         if time.monotonic() - start >= service.WORKER_FINISH_TIMEOUT:
@@ -412,6 +449,7 @@ def display_check_result(check_name: str, results: Dict[Union[str, Status], int]
 
 def display_internal_error(context: ExecutionContext, event: events.InternalError) -> None:
     click.secho(event.message, fg="red")
+    click.echo()
     if event.exception:
         if context.show_errors_tracebacks:
             message = (
@@ -432,25 +470,26 @@ def handle_initialized(context: ExecutionContext, event: events.Initialized) ->
     """Display information about the test session."""
     context.operations_count = cast(int, event.operations_count)  # INVARIANT: should not be `None`
     display_section_name("Schemathesis test session starts")
-    versions = (
-        f"platform {platform.system()} -- "
-        f"Python {platform.python_version()}, "
-        f"schemathesis-{__version__}, "
-        f"hypothesis-{metadata.version('hypothesis')}, "
-        f"hypothesis_jsonschema-{metadata.version('hypothesis_jsonschema')}, "
-        f"jsonschema-{metadata.version('jsonschema')}"
-    )
-    click.echo(versions)
-    click.echo(f"rootdir: {os.getcwd()}")
-    click.echo(f"Hypothesis: {context.hypothesis_settings.show_changed()}")
+    if context.verbosity > 0:
+        versions = (
+            f"platform {platform.system()} -- "
+            f"Python {platform.python_version()}, "
+            f"schemathesis-{__version__}, "
+            f"hypothesis-{metadata.version('hypothesis')}, "
+            f"hypothesis_jsonschema-{metadata.version('hypothesis_jsonschema')}, "
+            f"jsonschema-{metadata.version('jsonschema')}"
+        )
+        click.echo(versions)
+        click.echo(f"rootdir: {os.getcwd()}")
+        click.echo(f"Hypothesis: {context.hypothesis_settings.show_changed()}")
     if event.location is not None:
-        click.echo(f"Schema location: {event.location}")
-    click.echo(f"Base URL: {event.base_url}")
-    click.echo(f"Specification version: {event.specification_name}")
-    click.echo(f"Workers: {context.workers_num}")
+        click.secho(f"Schema location: {event.location}", bold=True)
+    click.secho(f"Base URL: {event.base_url}", bold=True)
+    click.secho(f"Specification version: {event.specification_name}", bold=True)
+    click.secho(f"Workers: {context.workers_num}", bold=True)
     click.secho(f"Collected API operations: {context.operations_count}", bold=True)
-    if context.service is not None:
-        click.secho("Schemathesis.io: ENABLED", bold=True)
+    if isinstance(context.report, ServiceReportContext):
+        click.secho("Report to Schemathesis.io: ENABLED", bold=True)
     if context.operations_count >= 1:
         click.echo()
 
@@ -495,6 +534,7 @@ def handle_finished(context: ExecutionContext, event: events.Finished) -> None:
 
 def handle_interrupted(context: ExecutionContext, event: events.Interrupted) -> None:
     click.echo()
+    context.is_interrupted = True
     display_section_name("KeyboardInterrupt", "!", bold=False)
 
 
