@@ -12,6 +12,7 @@ from .utils import GenericResponse, deprecated_property
 
 if TYPE_CHECKING:
     from .models import APIOperation, Case
+    from .schemas import BaseSchema
 
 
 @unique
@@ -171,6 +172,9 @@ class HookDispatcher:
                 f"Hook '{name}' takes {len(spec.signature.parameters)} arguments but {len(signature.parameters)} is defined"
             )
 
+    def collect_statistic(self) -> Dict[str, int]:
+        return {name: len(hooks) for name, hooks in self._hooks.items()}
+
     def get_all_by_name(self, name: str) -> List[Callable]:
         """Get a list of hooks registered for a name."""
         return self._hooks.get(name, [])
@@ -240,6 +244,11 @@ def before_load_schema(context: HookContext, raw_schema: Dict[str, Any]) -> None
     """Called before schema instance is created."""
 
 
+@HookDispatcher.register_spec([HookScope.GLOBAL])
+def after_load_schema(context: HookContext, schema: "BaseSchema") -> None:
+    """Called after schema instance is created."""
+
+
 @all_scopes
 def before_add_examples(context: HookContext, examples: List["Case"]) -> None:
     """Called before explicit examples are added to a test via `@example` decorator.
@@ -261,9 +270,32 @@ def add_case(context: HookContext, case: "Case", response: GenericResponse) -> O
     """
 
 
+@HookDispatcher.register_spec([HookScope.GLOBAL])
+def before_call(context: HookContext, case: "Case") -> None:
+    """Called before every network call in CLI tests.
+
+    Use cases:
+     - Modification of `case`. For example, adding some pre-determined value to its query string.
+     - Logging
+    """
+
+
+@HookDispatcher.register_spec([HookScope.GLOBAL])
+def after_call(context: HookContext, case: "Case", response: GenericResponse) -> None:
+    """Called after every network call in CLI tests.
+
+    Note that you need to modify the response in-place.
+
+    Use cases:
+     - Response post-processing, like modifying its payload.
+     - Logging
+    """
+
+
 GLOBAL_HOOK_DISPATCHER = HookDispatcher(scope=HookScope.GLOBAL)
 dispatch = GLOBAL_HOOK_DISPATCHER.dispatch
 get_all_by_name = GLOBAL_HOOK_DISPATCHER.get_all_by_name
+collect_statistic = GLOBAL_HOOK_DISPATCHER.collect_statistic
 register = GLOBAL_HOOK_DISPATCHER.register
 unregister = GLOBAL_HOOK_DISPATCHER.unregister
 unregister_all = GLOBAL_HOOK_DISPATCHER.unregister_all
