@@ -58,11 +58,12 @@ from .exceptions import (
     get_grouped_exception,
     get_timeout_error,
 )
-from .hooks import GLOBAL_HOOK_DISPATCHER, HookContext, HookDispatcher
+from .hooks import GLOBAL_HOOK_DISPATCHER, HookContext, HookDispatcher, dispatch
 from .parameters import Parameter, ParameterSet, PayloadAlternatives
 from .serializers import Serializer, SerializerContext
 from .types import Body, Cookies, FormData, Headers, NotSet, PathParameters, Query
 from .utils import (
+    IGNORED_HEADERS,
     NOT_SET,
     GenericResponse,
     WSGIResponse,
@@ -182,33 +183,6 @@ class Case:  # pylint: disable=too-many-public-methods
             return urlunsplit(("http", "localhost", path or "", "", ""))
         return self.base_url
 
-    def as_text_lines(self, headers: Optional[Dict[str, Any]] = None) -> List[str]:
-        """Textual representation.
-
-        Each component is a separate line.
-        """
-        output = {
-            "Path parameters": self.path_parameters,
-            "Headers": self.headers,
-            "Cookies": self.cookies,
-            "Query": self.query,
-            "Body": self.body,
-        }
-        if headers:
-            final_headers = output["Headers"] or {}
-            final_headers = cast(Dict[str, Any], final_headers)
-            final_headers.update(headers)
-            output["Headers"] = final_headers
-        max_length = max(map(len, output))
-        template = f"{{:<{max_length}}} : {{}}"
-
-        def should_display(key: str, value: Any) -> bool:
-            if key == "Body":
-                return not isinstance(value, NotSet)
-            return value is not None
-
-        return [template.format(key, value) for key, value in output.items() if should_display(key, value)]
-
     def get_code_to_reproduce(
         self, headers: Optional[Dict[str, Any]] = None, request: Optional[requests.PreparedRequest] = None
     ) -> str:
@@ -227,6 +201,7 @@ class Case:  # pylint: disable=too-many-public-methods
             final_headers = kwargs["headers"] or {}
             final_headers.update(headers)
             kwargs["headers"] = final_headers
+        kwargs["headers"] = {key: value for key, value in kwargs["headers"].items() if key not in IGNORED_HEADERS}
         method = kwargs["method"].lower()
 
         def should_display(key: str, value: Any) -> bool:
@@ -257,6 +232,9 @@ class Case:  # pylint: disable=too-many-public-methods
         if isinstance(prepared.body, bytes):
             # Note, it may be not sufficient to reproduce the error :(
             prepared.body = prepared.body.decode("utf-8", errors="replace")
+        for header in tuple(prepared.headers):
+            if header in IGNORED_HEADERS:
+                del prepared.headers[header]
         return curlify.to_curl(prepared)
 
     def _get_base_url(self, base_url: Optional[str] = None) -> str:
@@ -332,6 +310,8 @@ class Case:  # pylint: disable=too-many-public-methods
         **kwargs: Any,
     ) -> requests.Response:
         """Make a network call with `requests`."""
+        hook_context = HookContext(operation=self.operation)
+        dispatch("before_call", hook_context, self)
         data = self.as_requests_kwargs(base_url, headers)
         data.update(kwargs)
         data.setdefault("timeout", DEFAULT_RESPONSE_TIMEOUT / 1000)
@@ -350,6 +330,7 @@ class Case:  # pylint: disable=too-many-public-methods
                 f"\n\n1. Request timed out after {timeout:.2f}ms\n\n----------\n\n{code_message}",
                 context=failures.RequestTimeout(timeout=timeout),
             ) from None
+        dispatch("after_call", hook_context, self, response)
         if close_session:
             session.close()
         return response
@@ -383,12 +364,15 @@ class Case:  # pylint: disable=too-many-public-methods
                 "WSGI application instance is required. "
                 "Please, set `app` argument in the schema constructor or pass it to `call_wsgi`"
             )
+        hook_context = HookContext(operation=self.operation)
+        dispatch("before_call", hook_context, self)
         data = self.as_werkzeug_kwargs(headers)
         client = werkzeug.Client(application, WSGIResponse)
         with cookie_handler(client, self.cookies):
             response = client.open(**data, **kwargs)
         requests_kwargs = self.as_requests_kwargs(base_url=self.get_full_base_url(), headers=headers)
         response.request = requests.Request(**requests_kwargs).prepare()
+        dispatch("after_call", hook_context, self, response)
         return response
 
     def call_asgi(
@@ -954,7 +938,7 @@ class TestResult:
     method: str = attr.ib()  # pragma: no mutate
     path: str = attr.ib()  # pragma: no mutate
     verbose_name: str = attr.ib()  # pragma: no mutate
-    data_generation_method: DataGenerationMethod = attr.ib()  # pragma: no mutate
+    data_generation_method: List[DataGenerationMethod] = attr.ib()  # pragma: no mutate
     checks: List[Check] = attr.ib(factory=list)  # pragma: no mutate
     errors: List[Tuple[Exception, Optional[Case]]] = attr.ib(factory=list)  # pragma: no mutate
     interactions: List[Interaction] = attr.ib(factory=list)  # pragma: no mutate
@@ -1048,6 +1032,7 @@ class TestResultSet:
 
     results: List[TestResult] = attr.ib(factory=list)  # pragma: no mutate
     generic_errors: List[InvalidSchema] = attr.ib(factory=list)  # pragma: no mutate
+    warnings: List[str] = attr.ib(factory=list)  # pragma: no mutate
 
     def __iter__(self) -> Iterator[TestResult]:
         return iter(self.results)
@@ -1109,5 +1094,9 @@ class TestResultSet:
         """Add a new item to the results list."""
         self.results.append(item)
 
+    def add_warning(self, warning: str) -> None:
+        """Add a new warning to the warnings list."""
+        self.warnings.append(warning)
+
 
 CheckFunction = Callable[[GenericResponse, Case], Optional[bool]]  # pragma: no mutate
