@@ -5,11 +5,11 @@ from typing import Any, Dict, List, Optional, Union
 import attr
 from requests import exceptions
 
-from ..constants import DataGenerationMethod
+from ..constants import USE_WAIT_FOR_SCHEMA_SUGGESTION_MESSAGE, DataGenerationMethod
 from ..exceptions import HTTPError
 from ..models import APIOperation, Status, TestResult, TestResultSet
 from ..schemas import BaseSchema
-from ..utils import format_exception
+from ..utils import current_datetime, format_exception
 from .serialization import SerializedError, SerializedTestResult
 
 
@@ -40,18 +40,24 @@ class Initialized(ExecutionEvent):
     base_url: str = attr.ib()  # pragma: no mutate
     # API schema specification name
     specification_name: str = attr.ib()  # pragma: no mutate
-    # Timestamp of test run start
+    # Monotonic clock value when the test run started. Used to properly calculate run duration, since this clock
+    # can't go backwards.
     start_time: float = attr.ib(factory=time.monotonic)  # pragma: no mutate
+    # Datetime of the test run start
+    started_at: str = attr.ib(factory=current_datetime)  # pragma: no mutate
     thread_id: int = attr.ib(factory=threading.get_ident)  # pragma: no mutate
 
     @classmethod
-    def from_schema(cls, *, schema: BaseSchema, count_operations: bool = True) -> "Initialized":
+    def from_schema(
+        cls, *, schema: BaseSchema, count_operations: bool = True, started_at: Optional[str] = None
+    ) -> "Initialized":
         """Computes all needed data from a schema instance."""
         return cls(
             schema=schema.raw_schema,
             operations_count=schema.operations_count if count_operations else None,
             location=schema.location,
             base_url=schema.get_base_url(),
+            started_at=started_at or current_datetime(),
             specification_name=schema.verbose_name,
         )
 
@@ -83,8 +89,8 @@ class BeforeExecution(CurrentOperationMixin, ExecutionEvent):
     # The current level of recursion during stateful testing
     recursion_level: int = attr.ib()  # pragma: no mutate
     # The way data will be generated
-    data_generation_method: DataGenerationMethod = attr.ib()  # pragma: no mutate
-    # An unique ID which connects events that happen during testing of the same API operation
+    data_generation_method: List[DataGenerationMethod] = attr.ib()  # pragma: no mutate
+    # A unique ID which connects events that happen during testing of the same API operation
     # It may be useful when multiple threads are involved where incoming events are not ordered
     correlation_id: str = attr.ib()  # pragma: no mutate
     thread_id: int = attr.ib(factory=threading.get_ident)  # pragma: no mutate
@@ -94,7 +100,7 @@ class BeforeExecution(CurrentOperationMixin, ExecutionEvent):
         cls,
         operation: APIOperation,
         recursion_level: int,
-        data_generation_method: DataGenerationMethod,
+        data_generation_method: List[DataGenerationMethod],
         correlation_id: str,
     ) -> "BeforeExecution":
         return cls(
@@ -121,7 +127,7 @@ class AfterExecution(CurrentOperationMixin, ExecutionEvent):
     # APIOperation test status - success / failure / error
     status: Status = attr.ib()  # pragma: no mutate
     # The way data was generated
-    data_generation_method: DataGenerationMethod = attr.ib()  # pragma: no mutate
+    data_generation_method: List[DataGenerationMethod] = attr.ib()  # pragma: no mutate
     result: SerializedTestResult = attr.ib()  # pragma: no mutate
     # Test running time
     elapsed_time: float = attr.ib()  # pragma: no mutate
@@ -138,7 +144,7 @@ class AfterExecution(CurrentOperationMixin, ExecutionEvent):
         elapsed_time: float,
         hypothesis_output: List[str],
         operation: APIOperation,
-        data_generation_method: DataGenerationMethod,
+        data_generation_method: List[DataGenerationMethod],
         correlation_id: str,
     ) -> "AfterExecution":
         return cls(
@@ -175,7 +181,7 @@ class InternalError(ExecutionEvent):
     thread_id: int = attr.ib(factory=threading.get_ident)  # pragma: no mutate
 
     @classmethod
-    def from_exc(cls, exc: Exception) -> "InternalError":
+    def from_exc(cls, exc: Exception, wait_for_schema: Optional[float] = None) -> "InternalError":
         exception_type = f"{exc.__class__.__module__}.{exc.__class__.__qualname__}"
         if isinstance(exc, HTTPError):
             if exc.response.status_code == 404:
@@ -187,6 +193,8 @@ class InternalError(ExecutionEvent):
         exception_with_traceback = format_exception(exc, include_traceback=True)
         if isinstance(exc, exceptions.ConnectionError):
             message = f"Failed to load schema from {exc.request.url}"
+            if wait_for_schema is None:
+                message += f"\n{USE_WAIT_FOR_SCHEMA_SUGGESTION_MESSAGE}"
         else:
             message = "An internal error happened during a test run"
         return cls(
@@ -216,6 +224,7 @@ class Finished(ExecutionEvent):
     has_logs: bool = attr.ib()  # pragma: no mutate
     is_empty: bool = attr.ib()  # pragma: no mutate
     generic_errors: List[SerializedError] = attr.ib()  # pragma: no mutate
+    warnings: List[str] = attr.ib()  # pragma: no mutate
 
     total: Dict[str, Dict[Union[str, Status], int]] = attr.ib()  # pragma: no mutate
 
@@ -238,5 +247,6 @@ class Finished(ExecutionEvent):
             generic_errors=[
                 SerializedError.from_error(error, None, None, error.full_path) for error in results.generic_errors
             ],
+            warnings=results.warnings,
             running_time=running_time,
         )
