@@ -2,6 +2,7 @@
 import logging
 import threading
 import time
+import unittest
 import uuid
 from contextlib import contextmanager
 from types import TracebackType
@@ -17,6 +18,7 @@ from hypothesis_jsonschema._canonicalise import HypothesisRefResolutionError
 from requests.auth import HTTPDigestAuth, _basic_auth_str
 
 from ... import failures, hooks
+from ..._compat import MultipleFailures
 from ...auth import unregister as unregister_auth
 from ...constants import (
     DEFAULT_STATEFUL_RECURSION_LIMIT,
@@ -46,12 +48,17 @@ from ...utils import (
     WSGIResponse,
     capture_hypothesis_output,
     copy_response,
+    current_datetime,
     format_exception,
     maybe_set_assertion_message,
 )
 from ..serialization import SerializedTestResult
 
 
+def _should_count_towards_stop(event: events.ExecutionEvent) -> bool:
+    return isinstance(event, events.AfterExecution) and event.status in (Status.error, Status.failure)
+
+
 @attr.s  # pragma: no mutate
 class BaseRunner:
     schema: BaseSchema = attr.ib()  # pragma: no mutate
@@ -66,10 +73,13 @@ class BaseRunner:
     store_interactions: bool = attr.ib(default=False)  # pragma: no mutate
     seed: Optional[int] = attr.ib(default=None)  # pragma: no mutate
     exit_first: bool = attr.ib(default=False)  # pragma: no mutate
+    max_failures: Optional[int] = attr.ib(default=None)  # pragma: no mutate
+    started_at: str = attr.ib(factory=current_datetime)  # pragma: no mutate
     dry_run: bool = attr.ib(default=False)  # pragma: no mutate
     stateful: Optional[Stateful] = attr.ib(default=None)  # pragma: no mutate
     stateful_recursion_limit: int = attr.ib(default=DEFAULT_STATEFUL_RECURSION_LIMIT)  # pragma: no mutate
     count_operations: bool = attr.ib(default=True)  # pragma: no mutate
+    _failures_counter: int = attr.ib(default=0)
 
     def execute(self) -> "EventStream":
         """Common logic for all runners."""
@@ -85,6 +95,8 @@ class BaseRunner:
         initialized = events.Initialized.from_schema(schema=self.schema, count_operations=self.count_operations)
 
         def _finish() -> events.Finished:
+            if has_all_not_found(results):
+                results.add_warning(ALL_NOT_FOUND_WARNING_MESSAGE)
             return events.Finished.from_results(results=results, running_time=time.monotonic() - initialized.start_time)
 
         if stop_event.is_set():
@@ -106,11 +118,13 @@ class BaseRunner:
         yield _finish()
 
     def _should_stop(self, event: events.ExecutionEvent) -> bool:
-        return (
-            self.exit_first
-            and isinstance(event, events.AfterExecution)
-            and event.status in (Status.error, Status.failure)
-        )
+        if _should_count_towards_stop(event):
+            if self.exit_first:
+                return True
+            if self.max_failures is not None:
+                self._failures_counter += 1
+                return self._failures_counter >= self.max_failures
+        return False
 
     def _execute(
         self, results: TestResultSet, stop_event: threading.Event
@@ -130,7 +144,7 @@ class BaseRunner:
         """Run tests and recursively run additional tests."""
         if recursion_level > self.stateful_recursion_limit:
             return
-        for result, data_generation_method in maker(template, settings, seed):
+        for result in maker(template, settings, seed):
             if isinstance(result, Ok):
                 operation, test = result.ok()
                 feedback = Feedback(self.stateful, operation)
@@ -145,7 +159,7 @@ class BaseRunner:
                         results=results,
                         feedback=feedback,
                         recursion_level=recursion_level,
-                        data_generation_method=data_generation_method,
+                        data_generation_methods=self.schema.data_generation_methods,
                         **kwargs,
                     ):
                         yield event
@@ -167,13 +181,15 @@ class BaseRunner:
                     yield from handle_schema_error(
                         exc,
                         results,
-                        data_generation_method,
+                        self.schema.data_generation_methods,
                         recursion_level,
                         before_execution_correlation_id=before_execution_correlation_id,
                     )
             else:
                 # Schema errors
-                yield from handle_schema_error(result.err(), results, data_generation_method, recursion_level)
+                yield from handle_schema_error(
+                    result.err(), results, self.schema.data_generation_methods, recursion_level
+                )
 
 
 @attr.s(slots=True)  # pragma: no mutate
@@ -208,7 +224,7 @@ class EventStream:
 def handle_schema_error(
     error: InvalidSchema,
     results: TestResultSet,
-    data_generation_method: DataGenerationMethod,
+    data_generation_methods: Iterable[DataGenerationMethod],
     recursion_level: int,
     *,
     before_execution_correlation_id: Optional[str] = None,
@@ -216,13 +232,14 @@ def handle_schema_error(
     if error.method is not None:
         assert error.path is not None
         assert error.full_path is not None
+        data_generation_methods = list(data_generation_methods)
         method = error.method.upper()
         verbose_name = f"{method} {error.full_path}"
         result = TestResult(
             method=method,
             path=error.full_path,
             verbose_name=verbose_name,
-            data_generation_method=data_generation_method,
+            data_generation_method=data_generation_methods,
         )
         result.add_error(error)
         # It might be already emitted - reuse its correlation id
@@ -236,7 +253,7 @@ def handle_schema_error(
                 verbose_name=verbose_name,
                 relative_path=error.path,
                 recursion_level=recursion_level,
-                data_generation_method=data_generation_method,
+                data_generation_method=data_generation_methods,
                 correlation_id=correlation_id,
             )
         yield events.AfterExecution(
@@ -246,7 +263,7 @@ def handle_schema_error(
             verbose_name=verbose_name,
             status=Status.error,
             result=SerializedTestResult.from_test_result(result),
-            data_generation_method=data_generation_method,
+            data_generation_method=data_generation_methods,
             elapsed_time=0.0,
             hypothesis_output=[],
             correlation_id=correlation_id,
@@ -262,7 +279,7 @@ def run_test(  # pylint: disable=too-many-locals
     operation: APIOperation,
     test: Callable,
     checks: Iterable[CheckFunction],
-    data_generation_method: DataGenerationMethod,
+    data_generation_methods: Iterable[DataGenerationMethod],
     targets: Iterable[Target],
     results: TestResultSet,
     headers: Optional[Dict[str, Any]],
@@ -270,19 +287,20 @@ def run_test(  # pylint: disable=too-many-locals
     **kwargs: Any,
 ) -> Generator[events.ExecutionEvent, None, None]:
     """A single test run with all error handling needed."""
+    data_generation_methods = list(data_generation_methods)
     result = TestResult(
         method=operation.method.upper(),
         path=operation.full_path,
         verbose_name=operation.verbose_name,
         overridden_headers=headers,
-        data_generation_method=data_generation_method,
+        data_generation_method=data_generation_methods,
     )
     # To simplify connecting `before` and `after` events in external systems
     correlation_id = uuid.uuid4().hex
     yield events.BeforeExecution.from_operation(
         operation=operation,
         recursion_level=recursion_level,
-        data_generation_method=data_generation_method,
+        data_generation_method=data_generation_methods,
         correlation_id=correlation_id,
     )
     hypothesis_output: List[str] = []
@@ -298,6 +316,10 @@ def run_test(  # pylint: disable=too-many-locals
             result.mark_skipped()
         else:
             status = Status.success
+    except unittest.case.SkipTest:
+        # Newer Hypothesis versions raise this exception if no tests were executed
+        status = Status.skip
+        result.mark_skipped()
     except CheckFailed:
         status = Status.failure
     except NonCheckError:
@@ -306,9 +328,9 @@ def run_test(  # pylint: disable=too-many-locals
         result.mark_errored()
         for error in deduplicate_errors(errors):
             result.add_error(error)
-    except hypothesis.errors.MultipleFailures:
+    except MultipleFailures:
         # Schemathesis may detect multiple errors that come from different check results
-        # They raise different "grouped" exceptions, and `MultipleFailures` is risen as the result
+        # They raise different "grouped" exceptions
         status = Status.failure
     except hypothesis.errors.Flaky:
         status = Status.failure
@@ -351,17 +373,59 @@ def run_test(  # pylint: disable=too-many-locals
         test, "_hypothesis_internal_use_generated_seed", None
     )
     results.append(result)
+    for status_code in (401, 403):
+        if has_too_many_responses_with_status(result, status_code):
+            results.add_warning(TOO_MANY_RESPONSES_WARNING_TEMPLATE.format(f"`{operation.verbose_name}`", status_code))
     yield events.AfterExecution.from_result(
         result=result,
         status=status,
         elapsed_time=test_elapsed_time,
         hypothesis_output=hypothesis_output,
         operation=operation,
-        data_generation_method=data_generation_method,
+        data_generation_method=data_generation_methods,
         correlation_id=correlation_id,
     )
 
 
+TOO_MANY_RESPONSES_WARNING_TEMPLATE = (
+    "Most of the responses from {} have a {} status code. Did you specify proper API credentials?"
+)
+TOO_MANY_RESPONSES_THRESHOLD = 0.9
+
+
+def has_too_many_responses_with_status(result: TestResult, status_code: int) -> bool:
+    # It is faster than creating an intermediate list
+    unauthorized_count = 0
+    total = 0
+    for check in result.checks:
+        if check.response is not None:
+            if check.response.status_code == status_code:
+                unauthorized_count += 1
+            total += 1
+    if not total:
+        return False
+    return unauthorized_count / total >= TOO_MANY_RESPONSES_THRESHOLD
+
+
+ALL_NOT_FOUND_WARNING_MESSAGE = "All API responses have a 404 status code. Did you specify the proper API location?"
+
+
+def has_all_not_found(results: TestResultSet) -> bool:
+    """Check if all responses are 404."""
+    has_not_found = False
+    for result in results.results:
+        for check in result.checks:
+            if check.response is not None:
+                if check.response.status_code == 404:
+                    has_not_found = True
+                else:
+                    # There are non-404 responses, no reason to check any other response
+                    return False
+    # Only happens if all responses are 404, ot there are no responses at all.
+    # In the first case, it returns True, for the latter - False
+    return has_not_found
+
+
 def setup_hypothesis_database_key(test: Callable, operation: APIOperation) -> None:
     """Make Hypothesis use separate database entries for every API operation.
 
@@ -581,7 +645,6 @@ def _network_test(
     check_results: List[Check] = []
     try:
         hook_context = HookContext(operation=case.operation)
-        hooks.dispatch("before_call", hook_context, case)
         kwargs: Dict[str, Any] = {
             "session": session,
             "headers": headers,
@@ -591,7 +654,6 @@ def _network_test(
         }
         hooks.dispatch("process_call_kwargs", hook_context, case, kwargs)
         response = case.call(**kwargs)
-        hooks.dispatch("after_call", hook_context, case, response)
     except CheckFailed as exc:
         check_name = "request_timeout"
         requests_kwargs = case.as_requests_kwargs(base_url=case.get_full_base_url(), headers=headers)
@@ -681,11 +743,9 @@ def _wsgi_test(
     with catching_logs(LogCaptureHandler(), level=logging.DEBUG) as recorded:
         start = time.monotonic()
         hook_context = HookContext(operation=case.operation)
-        hooks.dispatch("before_call", hook_context, case)
         kwargs = {"headers": headers}
         hooks.dispatch("process_call_kwargs", hook_context, case, kwargs)
         response = case.call_wsgi(**kwargs)
-        hooks.dispatch("after_call", hook_context, case, response)
         elapsed = time.monotonic() - start
     context = TargetContext(case=case, response=response, response_time=elapsed)
     run_targets(targets, context)
@@ -770,11 +830,9 @@ def _asgi_test(
     max_response_time: Optional[int],
 ) -> requests.Response:
     hook_context = HookContext(operation=case.operation)
-    hooks.dispatch("before_call", hook_context, case)
     kwargs: Dict[str, Any] = {"headers": headers}
     hooks.dispatch("process_call_kwargs", hook_context, case, kwargs)
     response = case.call_asgi(**kwargs)
-    hooks.dispatch("after_call", hook_context, case, response)
     context = TargetContext(case=case, response=response, response_time=response.elapsed.total_seconds())
     run_targets(targets, context)
     status = Status.success
