@@ -3,19 +3,18 @@
 They all consist of primitive types and don't have references to schemas, app, etc.
 """
 import logging
-from typing import Any, Dict, Generator, List, Optional, Set, Tuple
+from typing import Any, Dict, List, Optional, Set, Tuple
 
 import attr
 import requests
 
 from ..exceptions import FailureContext, InternalError, make_unique_by_key
 from ..models import Case, Check, Interaction, Request, Response, Status, TestResult
-from ..utils import WSGIResponse, format_exception
+from ..utils import IGNORED_HEADERS, WSGIResponse, format_exception
 
 
 @attr.s(slots=True)  # pragma: no mutate
 class SerializedCase:
-    text_lines: List[str] = attr.ib()  # pragma: no mutate
     requests_code: str = attr.ib()
     curl_code: str = attr.ib()
     path_template: str = attr.ib()
@@ -23,12 +22,12 @@ class SerializedCase:
     query: Optional[Dict[str, Any]] = attr.ib()
     cookies: Optional[Dict[str, Any]] = attr.ib()
     verbose_name: str = attr.ib()
+    data_generation_method: Optional[str] = attr.ib()
     media_type: Optional[str] = attr.ib()
 
     @classmethod
     def from_case(cls, case: Case, headers: Optional[Dict[str, Any]]) -> "SerializedCase":
         return cls(
-            text_lines=case.as_text_lines(headers),
             requests_code=case.get_code_to_reproduce(headers),
             curl_code=case.as_curl_command(headers),
             path_template=case.path,
@@ -36,6 +35,9 @@ class SerializedCase:
             query=case.query,
             cookies=case.cookies,
             verbose_name=case.operation.verbose_name,
+            data_generation_method=case.data_generation_method.as_short_name()
+            if case.data_generation_method is not None
+            else None,
             media_type=case.media_type,
         )
 
@@ -73,7 +75,7 @@ class SerializedCheck:
             response = Response.from_wsgi(check.response, check.elapsed)
         else:
             response = None
-        headers = {key: value[0] for key, value in request.headers.items()}
+        headers = {key: value[0] for key, value in request.headers.items() if key not in IGNORED_HEADERS}
         history = []
         case = check.example
         while case.source is not None:
@@ -154,7 +156,7 @@ class SerializedTestResult:
     is_flaky: bool = attr.ib()  # pragma: no mutate
     is_skipped: bool = attr.ib()  # pragma: no mutate
     seed: Optional[int] = attr.ib()  # pragma: no mutate
-    data_generation_method: str = attr.ib()  # pragma: no mutate
+    data_generation_method: List[str] = attr.ib()  # pragma: no mutate
     checks: List[SerializedCheck] = attr.ib()  # pragma: no mutate
     logs: List[str] = attr.ib()  # pragma: no mutate
     errors: List[SerializedError] = attr.ib()  # pragma: no mutate
@@ -174,7 +176,7 @@ class SerializedTestResult:
             is_flaky=result.is_flaky,
             is_skipped=result.is_skipped,
             seed=result.seed,
-            data_generation_method=result.data_generation_method.as_short_name(),
+            data_generation_method=[m.as_short_name() for m in result.data_generation_method],
             checks=[SerializedCheck.from_check(check) for check in result.checks],
             logs=[formatter.format(record) for record in result.logs],
             errors=[SerializedError.from_error(*error, headers=result.overridden_headers) for error in result.errors],
@@ -194,13 +196,3 @@ def deduplicate_failures(checks: List[SerializedCheck]) -> List[SerializedCheck]
                 unique_checks.append(check)
                 seen.add(key)
     return unique_checks
-
-
-def deduplicate_checks(checks: List[SerializedCheck]) -> Generator[SerializedCheck, None, None]:
-    """Return only unique checks outcomes."""
-    seen: Set[Tuple[Optional[str], ...]] = set()
-    for check in reversed(checks):
-        key = make_unique_by_key(check.name, check.message, check.context)
-        if key not in seen:
-            yield check
-            seen.add(key)
