@@ -1,4 +1,4 @@
-from . import auth, hosts
+from . import auth, ci, hosts
 from .client import ServiceClient
 from .constants import (
     DEFAULT_HOSTNAME,
@@ -8,11 +8,12 @@ from .constants import (
     HOSTNAME_ENV_VAR,
     HOSTS_PATH_ENV_VAR,
     PROTOCOL_ENV_VAR,
+    REPORT_ENV_VAR,
+    TELEMETRY_ENV_VAR,
     TOKEN_ENV_VAR,
     URL_ENV_VAR,
     WORKER_CHECK_PERIOD,
     WORKER_FINISH_TIMEOUT,
 )
-from .events import Completed, Error, Event, Timeout
-from .handler import ServiceReporter
-from .models import TestRun
+from .events import Completed, Error, Event, Failed, Metadata, Timeout
+from .report import FileReportHandler, ServiceReportHandler
