@@ -1,6 +1,9 @@
+from typing import Optional
+
 import attr
 
 from ..utils import format_exception
+from . import ci
 
 
 class Event:
@@ -10,15 +13,24 @@ class Event:
     """
 
     @property
-    def name(self) -> str:
+    def status(self) -> str:
         return self.__class__.__name__.upper()
 
 
 @attr.s(slots=True)
-class Completed(Event):
-    """The handler finished successfully."""
+class Metadata(Event):
+    """Meta-information about the report."""
 
-    short_url: str = attr.ib()
+    size: int = attr.ib()
+    ci_environment: Optional[ci.Environment] = attr.ib()
+
+
+@attr.s(slots=True)
+class Completed(Event):
+    """Report uploaded successfully."""
+
+    message: str = attr.ib()
+    next_url: str = attr.ib()
 
 
 @attr.s(slots=True)
@@ -31,6 +43,13 @@ class Error(Event):
         return format_exception(self.exception, include_traceback=include_traceback)
 
 
+@attr.s(slots=True)
+class Failed(Event):
+    """A client-side error which should be displayed to the user."""
+
+    detail: str = attr.ib()
+
+
 @attr.s(slots=True)
 class Timeout(Event):
     """The handler did not finish its work in time.
