@@ -1,8 +1,10 @@
 """Work with stored auth data."""
 import enum
+import tempfile
 from pathlib import Path
 from typing import Any, Dict, Optional
 
+import attr
 import tomli
 import tomli_w
 
@@ -10,11 +12,34 @@ from ..types import PathLike
 from .constants import DEFAULT_HOSTNAME, DEFAULT_HOSTS_PATH, HOSTS_FORMAT_VERSION
 
 
+@attr.s(slots=True)
+class HostData:
+    """Stored data related to a host."""
+
+    hostname: str = attr.ib()
+    hosts_file: PathLike = attr.ib()
+
+    def load(self) -> Dict[str, Any]:
+        return load(self.hosts_file).get(self.hostname, {})
+
+    @property
+    def correlation_id(self) -> Optional[str]:
+        return self.load().get("correlation_id")
+
+    def store_correlation_id(self, correlation_id: str) -> None:
+        """Store `correlation_id` in the hosts file."""
+        hosts = load(self.hosts_file)
+        data = hosts.setdefault(self.hostname, {})
+        data["correlation_id"] = correlation_id
+        _dump_hosts(self.hosts_file, hosts)
+
+
 def store(token: str, hostname: str = DEFAULT_HOSTNAME, hosts_file: PathLike = DEFAULT_HOSTS_PATH) -> None:
     """Store a new token for a host."""
     # Don't use any file-based locking for simplicity
     hosts = load(hosts_file)
-    hosts[hostname] = {"version": HOSTS_FORMAT_VERSION, "token": token}
+    data = hosts.setdefault(hostname, {})
+    data.update(version=HOSTS_FORMAT_VERSION, token=token)
     _dump_hosts(hosts_file, hosts)
 
 
@@ -27,17 +52,26 @@ def load(path: PathLike) -> Dict[str, Any]:
         with open(path, "rb") as fd:
             return tomli.load(fd)
     except FileNotFoundError:
-        # Try to create the parent dir - it could be the first run, when the config dir doesn't exist yet
-        try:
-            Path(path).parent.mkdir(mode=0o755, parents=True, exist_ok=True)
-        except OSError:
-            # Ignore permission errors, etc
-            pass
+        _try_make_config_directory(path)
         return {}
     except tomli.TOMLDecodeError:
         return {}
 
 
+def _try_make_config_directory(path: PathLike) -> None:
+    # Try to create the parent dir - it could be the first run, when the config dir doesn't exist yet
+    try:
+        Path(path).parent.mkdir(mode=0o755, parents=True, exist_ok=True)
+    except OSError:
+        # Ignore permission errors, etc
+        pass
+
+
+def load_for_host(hostname: str = DEFAULT_HOSTNAME, hosts_file: PathLike = DEFAULT_HOSTS_PATH) -> Dict[str, Any]:
+    """Load all data associated with a hostname."""
+    return load(hosts_file).get(hostname, {})
+
+
 @enum.unique
 class RemoveAuth(enum.Enum):
     success = 1
@@ -65,7 +99,12 @@ def remove(hostname: str = DEFAULT_HOSTNAME, hosts_file: PathLike = DEFAULT_HOST
 
 def get_token(hostname: str = DEFAULT_HOSTNAME, hosts_file: PathLike = DEFAULT_HOSTS_PATH) -> Optional[str]:
     """Load a token for a host."""
-    return load(hosts_file).get(hostname, {}).get("token")
+    return load_for_host(hostname, hosts_file).get("token")
+
+
+def get_temporary_hosts_file() -> str:
+    temporary_dir = Path(tempfile.gettempdir()).resolve()
+    return str(temporary_dir / "schemathesis-hosts.toml")
 
 
 def _dump_hosts(path: PathLike, hosts: Dict[str, Any]) -> None:
