@@ -4,18 +4,23 @@ import attr
 
 
 @attr.s(slots=True)
-class ApiConfig:
+class ApiDetails:
     location: str = attr.ib()
     base_url: Optional[str] = attr.ib()
 
 
 @attr.s(slots=True)
-class TestRun:
-    run_id: str = attr.ib()
-    short_url: str = attr.ib()
-    config: ApiConfig = attr.ib()
+class AuthResponse:
+    username: str = attr.ib()
 
 
 @attr.s(slots=True)
-class AuthResponse:
-    username: str = attr.ib()
+class UploadResponse:
+    message: str = attr.ib()
+    next_url: str = attr.ib()
+    correlation_id: str = attr.ib()
+
+
+@attr.s(slots=True)
+class FailedUploadResponse:
+    detail: str = attr.ib()
