@@ -4,7 +4,7 @@ import attr
 
 from ..models import Response
 from ..runner import events
-from ..runner.serialization import SerializedCase, deduplicate_checks
+from ..runner.serialization import SerializedCase
 from ..utils import merge
 
 S = TypeVar("S", bound=events.ExecutionEvent)
@@ -53,6 +53,8 @@ def serialize_after_execution(event: events.AfterExecution) -> Optional[Dict[str
     return {
         "correlation_id": event.correlation_id,
         "verbose_name": event.verbose_name,
+        "status": event.status,
+        "elapsed_time": event.elapsed_time,
         "data_generation_method": event.data_generation_method,
         "result": {
             "checks": [
@@ -74,7 +76,7 @@ def serialize_after_execution(event: events.AfterExecution) -> Optional[Dict[str
                         for entry in check.history
                     ],
                 }
-                for check in deduplicate_checks(event.result.checks)
+                for check in event.result.checks
             ],
             "errors": [
                 {
@@ -109,7 +111,8 @@ def serialize_finished(event: events.Finished) -> Optional[Dict[str, Any]]:
                 "title": error.title,
             }
             for error in event.generic_errors
-        ]
+        ],
+        "running_time": event.running_time,
     }
 
 
