@@ -3,6 +3,7 @@ import pathlib
 from typing import IO, Any, Callable, Dict, List, Optional, Tuple, Union, cast
 from urllib.parse import urljoin
 
+import backoff
 import jsonschema
 import requests
 import yaml
@@ -12,7 +13,7 @@ from starlette.testclient import TestClient as ASGIClient
 from werkzeug.test import Client
 from yarl import URL
 
-from ...constants import DEFAULT_DATA_GENERATION_METHODS, CodeSampleStyle
+from ...constants import DEFAULT_DATA_GENERATION_METHODS, WAIT_FOR_SCHEMA_INTERVAL, CodeSampleStyle
 from ...exceptions import HTTPError, SchemaLoadingError
 from ...hooks import HookContext, dispatch
 from ...lazy import LazySchema
@@ -39,7 +40,7 @@ def from_path(
     tag: Optional[Filter] = None,
     operation_id: Optional[Filter] = None,
     skip_deprecated_operations: bool = False,
-    validate_schema: bool = True,
+    validate_schema: bool = False,
     force_schema_version: Optional[str] = None,
     data_generation_methods: DataGenerationMethodInput = DEFAULT_DATA_GENERATION_METHODS,
     code_sample_style: str = CodeSampleStyle.default().name,
@@ -79,10 +80,11 @@ def from_uri(
     tag: Optional[Filter] = None,
     operation_id: Optional[Filter] = None,
     skip_deprecated_operations: bool = False,
-    validate_schema: bool = True,
+    validate_schema: bool = False,
     force_schema_version: Optional[str] = None,
     data_generation_methods: DataGenerationMethodInput = DEFAULT_DATA_GENERATION_METHODS,
     code_sample_style: str = CodeSampleStyle.default().name,
+    wait_for_schema: Optional[float] = None,
     **kwargs: Any,
 ) -> BaseOpenAPISchema:
     """Load Open API schema from the network.
@@ -92,7 +94,22 @@ def from_uri(
     setup_headers(kwargs)
     if not base_url and port:
         base_url = str(URL(uri).with_port(port))
-    response = requests.get(uri, **kwargs)
+
+    if wait_for_schema is not None:
+
+        @backoff.on_exception(  # type: ignore
+            backoff.constant,
+            requests.exceptions.ConnectionError,
+            max_time=wait_for_schema,
+            interval=WAIT_FOR_SCHEMA_INTERVAL,
+        )
+        def _load_schema(_uri: str, **_kwargs: Any) -> requests.Response:
+            return requests.get(_uri, **kwargs)
+
+    else:
+        _load_schema = requests.get
+
+    response = _load_schema(uri, **kwargs)
     HTTPError.raise_for_status(response)
     try:
         return from_file(
@@ -133,7 +150,7 @@ def from_file(
     tag: Optional[Filter] = None,
     operation_id: Optional[Filter] = None,
     skip_deprecated_operations: bool = False,
-    validate_schema: bool = True,
+    validate_schema: bool = False,
     force_schema_version: Optional[str] = None,
     data_generation_methods: DataGenerationMethodInput = DEFAULT_DATA_GENERATION_METHODS,
     code_sample_style: str = CodeSampleStyle.default().name,
@@ -175,7 +192,7 @@ def from_dict(
     tag: Optional[Filter] = None,
     operation_id: Optional[Filter] = None,
     skip_deprecated_operations: bool = False,
-    validate_schema: bool = True,
+    validate_schema: bool = False,
     force_schema_version: Optional[str] = None,
     data_generation_methods: DataGenerationMethodInput = DEFAULT_DATA_GENERATION_METHODS,
     code_sample_style: str = CodeSampleStyle.default().name,
@@ -186,11 +203,12 @@ def from_dict(
     :param dict raw_schema: A schema to load.
     """
     _code_sample_style = CodeSampleStyle.from_str(code_sample_style)
-    dispatch("before_load_schema", HookContext(), raw_schema)
+    hook_context = HookContext()
+    dispatch("before_load_schema", hook_context, raw_schema)
 
     def init_openapi_2() -> SwaggerV20:
         _maybe_validate_schema(raw_schema, definitions.SWAGGER_20_VALIDATOR, validate_schema)
-        return SwaggerV20(
+        instance = SwaggerV20(
             raw_schema,
             app=app,
             base_url=base_url,
@@ -204,10 +222,12 @@ def from_dict(
             code_sample_style=_code_sample_style,
             location=location,
         )
+        dispatch("after_load_schema", hook_context, instance)
+        return instance
 
     def init_openapi_3() -> OpenApi30:
         _maybe_validate_schema(raw_schema, definitions.OPENAPI_30_VALIDATOR, validate_schema)
-        return OpenApi30(
+        instance = OpenApi30(
             raw_schema,
             app=app,
             base_url=base_url,
@@ -221,6 +241,8 @@ def from_dict(
             code_sample_style=_code_sample_style,
             location=location,
         )
+        dispatch("after_load_schema", hook_context, instance)
+        return instance
 
     if force_schema_version == "20":
         return init_openapi_2()
@@ -282,7 +304,7 @@ def from_pytest_fixture(
     tag: Optional[Filter] = NOT_SET,
     operation_id: Optional[Filter] = NOT_SET,
     skip_deprecated_operations: bool = False,
-    validate_schema: bool = True,
+    validate_schema: bool = False,
     data_generation_methods: Union[DataGenerationMethodInput, NotSet] = NOT_SET,
     code_sample_style: str = CodeSampleStyle.default().name,
 ) -> LazySchema:
@@ -327,7 +349,7 @@ def from_wsgi(
     tag: Optional[Filter] = None,
     operation_id: Optional[Filter] = None,
     skip_deprecated_operations: bool = False,
-    validate_schema: bool = True,
+    validate_schema: bool = False,
     force_schema_version: Optional[str] = None,
     data_generation_methods: DataGenerationMethodInput = DEFAULT_DATA_GENERATION_METHODS,
     code_sample_style: str = CodeSampleStyle.default().name,
@@ -378,7 +400,7 @@ def from_aiohttp(
     tag: Optional[Filter] = None,
     operation_id: Optional[Filter] = None,
     skip_deprecated_operations: bool = False,
-    validate_schema: bool = True,
+    validate_schema: bool = False,
     force_schema_version: Optional[str] = None,
     data_generation_methods: DataGenerationMethodInput = DEFAULT_DATA_GENERATION_METHODS,
     code_sample_style: str = CodeSampleStyle.default().name,
@@ -420,7 +442,7 @@ def from_asgi(
     tag: Optional[Filter] = None,
     operation_id: Optional[Filter] = None,
     skip_deprecated_operations: bool = False,
-    validate_schema: bool = True,
+    validate_schema: bool = False,
     force_schema_version: Optional[str] = None,
     data_generation_methods: DataGenerationMethodInput = DEFAULT_DATA_GENERATION_METHODS,
     code_sample_style: str = CodeSampleStyle.default().name,
