@@ -3,6 +3,8 @@ from typing import Any, ClassVar, Dict, Iterable, List, Optional, Tuple
 
 import attr
 
+from ...exceptions import InvalidSchema
+from ...models import APIOperation
 from ...parameters import Parameter
 from .converter import to_json_schema_recursive
 
@@ -86,9 +88,9 @@ class OpenAPIParameter(Parameter):
         """
         return self.definition.get("schema", {}).get("example")
 
-    def as_json_schema(self) -> Dict[str, Any]:
+    def as_json_schema(self, operation: APIOperation) -> Dict[str, Any]:
         """Convert parameter's definition to JSON Schema."""
-        schema = self.from_open_api_to_json_schema(self.definition)
+        schema = self.from_open_api_to_json_schema(operation, self.definition)
         return self.transform_keywords(schema)
 
     def transform_keywords(self, schema: Dict[str, Any]) -> Dict[str, Any]:
@@ -105,7 +107,7 @@ class OpenAPIParameter(Parameter):
             definition.setdefault("type", "string")
         return definition
 
-    def from_open_api_to_json_schema(self, open_api_schema: Dict[str, Any]) -> Dict[str, Any]:
+    def from_open_api_to_json_schema(self, operation: APIOperation, open_api_schema: Dict[str, Any]) -> Dict[str, Any]:
         """Convert Open API's `Schema` to JSON Schema."""
         return {
             key: value
@@ -114,10 +116,10 @@ class OpenAPIParameter(Parameter):
             if key in self.supported_jsonschema_keywords or key.startswith("x-") or key == self.nullable_field
         }
 
-    def serialize(self) -> str:
+    def serialize(self, operation: APIOperation) -> str:
         # For simplicity, JSON Schema semantics is not taken into account (e.g. 1 == 1.0)
         # I.e. two semantically equal schemas may have different representation
-        return json.dumps(self.as_json_schema(), sort_keys=True)
+        return json.dumps(self.as_json_schema(operation), sort_keys=True)
 
 
 @attr.s(slots=True, eq=False)
@@ -210,9 +212,9 @@ class OpenAPI30Parameter(OpenAPIParameter):
     def is_header(self) -> bool:
         return self.location in ("header", "cookie")
 
-    def from_open_api_to_json_schema(self, open_api_schema: Dict[str, Any]) -> Dict[str, Any]:
-        open_api_schema = get_parameter_schema(open_api_schema)
-        return super().from_open_api_to_json_schema(open_api_schema)
+    def from_open_api_to_json_schema(self, operation: APIOperation, open_api_schema: Dict[str, Any]) -> Dict[str, Any]:
+        open_api_schema = get_parameter_schema(operation, open_api_schema)
+        return super().from_open_api_to_json_schema(operation, open_api_schema)
 
 
 @attr.s(slots=True, eq=False)
@@ -262,7 +264,7 @@ class OpenAPI20Body(OpenAPIBody, OpenAPI20Parameter):
     # NOTE. For Open API 2.0 bodies, we still give `x-example` precedence over the schema-level `example` field to keep
     # the precedence rules consistent.
 
-    def as_json_schema(self) -> Dict[str, Any]:
+    def as_json_schema(self, operation: APIOperation) -> Dict[str, Any]:
         """Convert body definition to JSON Schema."""
         # `schema` is required in Open API 2.0 when the `in` keyword is `body`
         schema = self.definition["schema"]
@@ -291,7 +293,7 @@ class OpenAPI30Body(OpenAPIBody, OpenAPI30Parameter):
     required: bool = attr.ib(default=False)
     description: Optional[str] = attr.ib(default=None)
 
-    def as_json_schema(self) -> Dict[str, Any]:
+    def as_json_schema(self, operation: APIOperation) -> Dict[str, Any]:
         """Convert body definition to JSON Schema."""
         schema = get_media_type_schema(self.definition)
         return self.transform_keywords(schema)
@@ -343,12 +345,12 @@ class OpenAPI20CompositeBody(OpenAPIBody, OpenAPI20Parameter):
     def _schema_example(self) -> Any:
         return {parameter.name: parameter._schema_example for parameter in self.definition if parameter._schema_example}
 
-    def as_json_schema(self) -> Dict[str, Any]:
+    def as_json_schema(self, operation: APIOperation) -> Dict[str, Any]:
         """The composite body is transformed into an "object" JSON Schema."""
-        return parameters_to_json_schema(self.definition)
+        return parameters_to_json_schema(operation, self.definition)
 
 
-def parameters_to_json_schema(parameters: Iterable[OpenAPIParameter]) -> Dict[str, Any]:
+def parameters_to_json_schema(operation: APIOperation, parameters: Iterable[OpenAPIParameter]) -> Dict[str, Any]:
     """Create an "object" JSON schema from a list of Open API parameters.
 
     :param List[OpenAPIParameter] parameters: A list of Open API parameters related to the same location. All of
@@ -388,20 +390,35 @@ def parameters_to_json_schema(parameters: Iterable[OpenAPIParameter]) -> Dict[st
     required = []
     for parameter in parameters:
         name = parameter.name
-        properties[name] = parameter.as_json_schema()
+        properties[name] = parameter.as_json_schema(operation)
         if parameter.is_required:
             required.append(name)
     return {"properties": properties, "additionalProperties": False, "type": "object", "required": required}
 
 
-def get_parameter_schema(data: Dict[str, Any]) -> Dict[str, Any]:
+MISSING_SCHEMA_OR_CONTENT_MESSAGE = (
+    'Can not generate data for {location} parameter "{name}"! '
+    "It should have either `schema` or `content` keywords defined"
+)
+
+
+def get_parameter_schema(operation: APIOperation, data: Dict[str, Any]) -> Dict[str, Any]:
     """Extract `schema` from Open API 3.0 `Parameter`."""
     # In Open API 3.0, there could be "schema" or "content" field. They are mutually exclusive.
     if "schema" in data:
         return data["schema"]
     # https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.0.3.md#fixed-fields-10
     # > The map MUST only contain one entry.
-    options = iter(data["content"].values())
+    try:
+        content = data["content"]
+    except KeyError as exc:
+        raise InvalidSchema(
+            MISSING_SCHEMA_OR_CONTENT_MESSAGE.format(location=data.get("in", ""), name=data.get("name", "<UNKNOWN>")),
+            path=operation.path,
+            method=operation.method,
+            full_path=operation.full_path,
+        ) from exc
+    options = iter(content.values())
     media_type_object = next(options)
     return get_media_type_schema(media_type_object)
 
