@@ -1,5 +1,6 @@
 import cgi
 import functools
+import operator
 import pathlib
 import re
 import sys
@@ -7,6 +8,7 @@ import traceback
 import warnings
 from contextlib import contextmanager
 from copy import copy, deepcopy
+from datetime import datetime, timezone
 from inspect import getfullargspec
 from json import JSONDecodeError
 from typing import (
@@ -30,17 +32,18 @@ import pytest
 import requests
 import yaml
 import yarl
+from hypothesis import strategies as st
 from hypothesis.core import is_invalid_test
 from hypothesis.reporting import with_reporter
 from hypothesis.strategies import SearchStrategy
 from requests.auth import HTTPDigestAuth
 from requests.exceptions import InvalidHeader  # type: ignore
-from requests.utils import check_header_validity  # type: ignore
+from requests.utils import CaseInsensitiveDict, check_header_validity, default_headers  # type: ignore
 from werkzeug.wrappers import Response as BaseResponse
 
-from ._compat import InferType, JSONMixin
+from ._compat import InferType, JSONMixin, get_signature
 from .constants import USER_AGENT, DataGenerationMethod
-from .exceptions import UsageError
+from .exceptions import SkipTest, UsageError
 from .types import DataGenerationMethodInput, Filter, GenericTest, NotSet, RawAuth
 
 try:
@@ -53,6 +56,13 @@ except ImportError:
 NOT_SET = NotSet()
 
 
+# These headers are added automatically by Schemathesis or `requests`.
+# Do not show them in code samples to make them more readable
+IGNORED_HEADERS = CaseInsensitiveDict(
+    {"Content-Length": None, "Transfer-Encoding": None, "Content-Type": None, **default_headers()}  # type: ignore
+)
+
+
 def file_exists(path: str) -> bool:
     try:
         return pathlib.Path(path).is_file()
@@ -442,8 +452,8 @@ def merge_given_args(func: GenericTest, args: Tuple, kwargs: Dict[str, Any]) ->
 
 
 def validate_given_args(func: GenericTest, args: Tuple, kwargs: Dict[str, Any]) -> Optional[Callable]:
-    argspec = getfullargspec(func)
-    return is_invalid_test(func, argspec, args, kwargs)  # type: ignore
+    signature = get_signature(func)
+    return is_invalid_test(func, signature, args, kwargs)  # type: ignore
 
 
 def compose(*functions: Callable) -> Callable:
@@ -480,3 +490,19 @@ def merge(a: Dict[str, Any], b: Dict[str, Any]) -> Dict[str, Any]:
         else:
             a[key] = b[key]
     return a
+
+
+def current_datetime() -> str:
+    return datetime.now(timezone.utc).astimezone().isoformat(timespec="seconds")
+
+
+def combine_strategies(strategies: List[st.SearchStrategy]) -> st.SearchStrategy:
+    """Combine a list of strategies into a single one.
+
+    If the input is `[a, b, c]`, then the result is equivalent to `a | b | c`.
+    """
+    return functools.reduce(operator.or_, strategies[1:], strategies[0])
+
+
+def skip(operation_name: str) -> NoReturn:
+    raise SkipTest(f"It is not possible to generate negative test cases for `{operation_name}`")
